#
# Copyright (c) 2008-2015 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'


# import standard library

from pyramid.events import subscriber
from pyramid.threadlocal import get_current_registry
from zope.interface import implementer
from zope.lifecycleevent import ObjectModifiedEvent
from zope.lifecycleevent.interfaces import IObjectAddedEvent, IObjectModifiedEvent, IObjectRemovedEvent
from zope.location import locate
from zope.location.interfaces import ISublocations
from zope.schema.fieldproperty import FieldProperty
from zope.traversing.interfaces import ITraversable

# import packages
from pyams_catalog.utils import index_object
from pyams_content import _
# import interfaces
from pyams_content.component.gallery.interfaces import IBaseGallery, IGallery, IGalleryTarget, IGalleryFile, \
    GALLERY_CONTAINER_KEY, GALLERY_RENDERERS
from pyams_content.component.paragraph import IBaseParagraph
from pyams_content.features.checker import BaseContentChecker
from pyams_content.features.checker.interfaces import IContentChecker
from pyams_content.features.renderer import RenderedContentMixin, RenderersVocabulary
from pyams_content.shared.common.interfaces import IWfSharedContent
from pyams_form.interfaces.form import IFormContextPermissionChecker
from pyams_i18n.interfaces import II18n
from pyams_utils.adapter import adapter_config, ContextAdapter, get_annotation_adapter
from pyams_utils.container import BTreeOrderedContainer
from pyams_utils.factory import factory_config
from pyams_utils.traversing import get_parent
from pyams_utils.vocabulary import vocabulary_config


#
# Galleries container
#

@implementer(IGallery)
@factory_config(provided=IBaseGallery)
class Gallery(RenderedContentMixin, BTreeOrderedContainer):
    """Gallery persistent class"""

    title = FieldProperty(IGallery['title'])
    description = FieldProperty(IGallery['description'])
    renderer = FieldProperty(IGallery['renderer'])

    last_id = 1

    def append(self, value, notify=True):
        key = str(self.last_id)
        if not notify:
            # pre-locate gallery item to avoid multiple notifications
            locate(value, self.key)
        self[key] = value
        self.last_id += 1
        if not notify:
            # make sure that gallery item is correctly indexed
            index_object(value)

    def get_visible_medias(self):
        yield from filter(lambda x: IGalleryFile(x).visible, self.values())


@adapter_config(context=IGalleryTarget, provides=IGallery)
def gallery_factory(target):
    """Galleries container factory"""
    return get_annotation_adapter(target, GALLERY_CONTAINER_KEY, Gallery, name='++gallery++')


@adapter_config(name='gallery', context=IGalleryTarget, provides=ITraversable)
class GalleryContainerNamespace(ContextAdapter):
    """++gallery++ namespace traverser"""

    def traverse(self, name, furtherpath=None):
        return IGallery(self.context)


@adapter_config(name='gallery', context=IGalleryTarget, provides=ISublocations)
class GalleryContainerSublocations(ContextAdapter):
    """Galleries container sublocations"""

    def sublocations(self):
        return IGallery(self.context).values()


@adapter_config(context=IGallery, provides=IFormContextPermissionChecker)
class GalleryPermissionChecker(ContextAdapter):
    """Gallery permission checker"""

    @property
    def edit_permission(self):
        content = get_parent(self.context, IWfSharedContent)
        return IFormContextPermissionChecker(content).edit_permission


@subscriber(IObjectAddedEvent, context_selector=IGallery)
def handle_added_gallery(event):
    """Handle added gallery"""
    gallery = event.object
    if IBaseParagraph.providedBy(gallery):
        # there is another event subscriber for paragraphs,
        # so don't trigger event twice !
        return
    content = get_parent(gallery, IWfSharedContent)
    if content is not None:
        get_current_registry().notify(ObjectModifiedEvent(content))


@subscriber(IObjectModifiedEvent, context_selector=IGallery)
def handle_modified_gallery(event):
    """Handle modified gallery"""
    gallery = event.object
    if IBaseParagraph.providedBy(gallery):
        # there is another event subscriber for paragraphs,
        # so don't trigger event twice !
        return
    content = get_parent(gallery, IWfSharedContent)
    if content is not None:
        get_current_registry().notify(ObjectModifiedEvent(content))


@subscriber(IObjectRemovedEvent, context_selector=IGallery)
def handle_removed_gallery(event):
    """Handle removed gallery"""
    gallery = event.object
    if IBaseParagraph.providedBy(gallery):
        # there is another event subscriber for paragraphs,
        # so don't trigger event twice !
        return
    content = get_parent(gallery, IWfSharedContent)
    if content is not None:
        get_current_registry().notify(ObjectModifiedEvent(content))


@adapter_config(name='gallery', context=IGallery, provides=IContentChecker)
class GalleryContentChecker(BaseContentChecker):
    """Gallery content checker"""

    label = _("Gallery")
    sep = '\n'
    weight = 60

    def inner_check(self, request):
        output = []
        registry = request.registry
        for media in IGallery(self.context).values():
            if not media.visible:
                continue
            for name, checker in sorted(registry.getAdapters((media, ), IContentChecker),
                                        key=lambda x: x[1].weight):
                output.append('- {0} : '.format(checker.label or
                                                II18n(media).query_attribute('title', request=request)))
                output.append(checker.get_check_output(request))
        return output


@adapter_config(name='gallery', context=IGalleryTarget, provides=IContentChecker)
def gallery_target_content_checker(context):
    gallery = IGallery(context)
    return IContentChecker(gallery, None)


@vocabulary_config(name=GALLERY_RENDERERS)
class GalleryRendererVocabulary(RenderersVocabulary):
    """Gallery renderers vocabulary"""

    content_interface = IBaseGallery
